@@ -22,6 +22,10 @@
 
 ;;; Code:
 
+;; TODO: Consider deleting the list of links as a separate list at all, and
+;; instead store list of links as one of the entry's fields.  That is
+;; consistent with how we store active timestamps and keywords too.
+
 (eval-when-compile
   (require 'cl-lib)
   (require 'subr-x))
@@ -36,19 +40,27 @@
 (defvar $use-tag-inheritance)
 (defvar $ignore-regions-regexps)
 
-(defun org-mem-parser--make-todo-regexp (keywords-string)
-  "Build a regexp from KEYWORDS-STRING.
-KEYWORDS-STRING is expected to be the sort of thing you see after
+(defun org-mem-parser--make-todo-regexp (s)
+  "Build a regexp from keyword string S.
+S is expected to be the sort of thing you see after
 a #+todo: or #+seq_todo: or #+typ_todo: setting in an Org file.
 
 The resulting regexp should be able to match any of
 the custom TODO words thus defined."
-  (thread-last keywords-string
-               (replace-regexp-in-string "(.*?)" "")
-               (string-replace "|" "")
-               (string-trim)
-               (split-string)
-               (regexp-opt)))
+  (let* ((words)
+         (pos (string-search "[ ]" s)))
+    (when pos
+      ;; Special-case Doom's [ ] keyword.
+      ;; https://github.com/orgs/doomemacs/discussions/74
+      (setq s (concat (substring s 0 pos)
+                      (substring s (+ 3 pos)))))
+    (setq words (thread-last s
+                             (replace-regexp-in-string "(.*?)" "")
+                             (string-replace "|" "")
+                             (string-trim)
+                             (split-string)))
+    (when pos (push "[ ]" words))
+    (regexp-opt words)))
 
 (defun org-mem-parser--mk-id (file-attribs entry-string)
   "Return a bignum that represents an entry.
@@ -114,6 +126,7 @@ brackets."
 
 ;;; Links, citations and active timestamps
 
+(defvar org-mem-parser--found-keywords nil)
 (defvar org-mem-parser--found-links nil)
 (defvar org-mem-parser--found-active-stamps nil)
 (defconst org-mem-parser--org-ts-regexp
@@ -257,7 +270,13 @@ the subheading potentially has an ID of its own."
     (goto-char beg)
     (while (re-search-forward org-mem-parser--org-ts-regexp end t)
       (push (org-mem-parser--time-string-to-int (match-string 0))
-            org-mem-parser--found-active-stamps)))
+            org-mem-parser--found-active-stamps))
+
+    ;; Start over and look for #+keywords
+    (goto-char beg)
+    (setq org-mem-parser--found-keywords
+          (nconc (org-mem-parser--collect-keywords end)
+                 org-mem-parser--found-keywords)))
   (goto-char (or end (point-max))))
 
 
@@ -291,6 +310,28 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
       (forward-line 1))
     alist))
 
+
+;;; Keywords
+
+(defconst org-mem-parser--org-keyword-regexp "^[ 	]*#\\+\\(\\S-+?\\):[ 	]*\\(.*\\)$"
+  "Copy of `org-keyword-regexp'.")
+
+(defun org-mem-parser--merge-same-keywords (keywords)
+  "In alist KEYWORDS, de-dup the cars by appending the cdrs."
+  (let (new-alist)
+    (cl-loop for (key . value) in keywords
+             do (if-let* ((cell (assoc key new-alist)))
+                    (setcdr cell (append value (cdr cell)))
+                  (push (cons key value) new-alist)))
+    (cl-loop for cell in new-alist do (setcdr cell (reverse (cdr cell))))
+    (nreverse new-alist)))
+
+(defun org-mem-parser--collect-keywords (end)
+  "Search for #+KEYWORDS: until position END, and return an alist."
+  (cl-loop
+   while (re-search-forward org-mem-parser--org-keyword-regexp end t)
+   collect (cons (upcase (match-string 1)) (list (match-string 2)))))
+
 
 ;;; Main
 
@@ -378,7 +419,7 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
             (insert-file-contents file)
             (setq coding-system last-coding-system-used)
             ;; Better get the attributes now at the same time we read the file,
-            ;; in the off-chance it gets altered in the time between parsing
+            ;; on the off-chance it gets altered in the time between parsing
             ;; it and getting its attributes.
             (setq file-attr (file-attributes file 'integer)))
 
@@ -424,11 +465,6 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
             (save-excursion
               (when (re-search-forward org-mem-parser--outline-regexp nil t)
                 (narrow-to-region (point-min) (pos-bol))))
-            ;; A bug introduced in org-node 5035a33 (fixed ~5 days later)
-            ;; could insert BACKLINKS before PROPERTIES.  Add a warning so
-            ;; user can fix the affected notes.
-            (when (looking-at-p "^[ \t]*:BACKLINKS:[ \t]*$")
-              (error "Code 12: Found BACKLINKS drawer before PROPERTIES \(likely inserted by org-node 3.4.3, bug fixed in 3.4.4)"))
             ;; We can safely assume that if there's a properties drawer,
             ;; it's the first drawer AND it comes before any #+keyword, at
             ;; least going by the behavior of `org-id-get'.
@@ -454,6 +490,9 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
               (forward-line))
             (setq RIGHT (point)) ;; End of the "front matter".
 
+            ;; TODO: Instead of searching manually for #+title & #+filetags,
+            ;; use the info from `org-mem-parser--found-keywords' (which is
+            ;; populated by `org-mem-parser--scan-visible-text').
             (goto-char LEFT)
             (when (re-search-forward "^#\\+FILETAGS:" RIGHT t)
               (when (not (eolp))
@@ -514,13 +553,14 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
                         nil
                         TAGS
                         nil
-                        PSEUDO-ID)
+                        PSEUDO-ID
+                        (org-mem-parser--merge-same-keywords
+                         org-mem-parser--found-keywords))
                 found-entries)
 
           ;; Prep
           (unless CRUMBS
             (push (list 0 1 1 nil nil nil nil) CRUMBS))
-          (setq org-mem-parser--found-active-stamps nil)
           (setq LNUM (line-number-at-pos))
 
           ;; Loop over the file's headings
@@ -535,6 +575,7 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
                    (pos-bol)
                  (point-max))))
             (setq org-mem-parser--found-active-stamps nil)
+            (setq org-mem-parser--found-keywords nil)
             (setq STATS-COOKIES nil)
             (setq INITIAL-STATS-COOKIES nil)
             (setq CLOCK-LINES nil)
@@ -726,7 +767,9 @@ between buffer substrings \":PROPERTIES:\" and \":END:\"."
                                      append (cl-sixth crumb))))
                           TAGS
                           TODO-STATE
-                          PSEUDO-ID)
+                          PSEUDO-ID
+                          (org-mem-parser--merge-same-keywords
+                           org-mem-parser--found-keywords))
                   found-entries)
             (goto-char (point-max))
             ;; NOTE: Famously slow `line-number-at-pos' fast in narrow region.
