@@ -20,13 +20,6 @@
 ;; Optional mechanisms to update the tables in just-in-time fashion,
 ;; reducing our need to do `org-mem--full-scan' so often.
 
-;; Technically, repeating a full scan is never needed *IF* we use these hooks
-;; correctly.  However, that is hard and humans are fallible.
-
-;; If a full scan is sufficiently performant, you can just do it more often,
-;; instead of using these hooks at all.  It is also a simple way to detect
-;; filesystem changes made by other Emacsen or the command line.
-
 ;;; Code:
 
 (require 'cl-lib)
@@ -40,37 +33,57 @@
 
 ;;; Targeted-scan
 
-(defun org-mem-updater-update (&optional synchronous)
+(defun org-mem-updater-update (&optional synchronous files)
   "Update cache for each file that has changed, appeared or disappeared.
 
 If SYNCHRONOUS, block Emacs until done.
-If SYNCHRONOUS and interrupted by a quit, cancel the update."
-  (let* ((db-files (copy-hash-table org-mem--truename<>metadata))
-         (modified-files
-          (cl-loop for truename in (org-mem--list-files-from-fs)
-                   as attr = (file-attributes truename)
-                   as real-mtime = (and attr (file-attribute-modification-time attr))
+If interrupted by a quit while blocking, cancel the update.
+
+If FILES, scan only FILES specifically.
+This is a hack you should almost never need, but can be used to sidestep
+rare performance issues with scanning the filesystem.
+
+If you must pass FILES, be sure to include the true name of every file
+touched by what you were doing, such as refiling an Org subtree from one
+file into another, or renaming one file name to another.
+Do not use `buffer-file-truename' for this, because it is a lie.
+Even \(expand-file-name buffer-file-truename) may not be the true name.
+
+Be sure also to include the old true name of any files that have been
+deleted or renamed, so that they may be removed from org-mem tables.
+If you mess up the tables, use command `org-mem-reset'."
+  (unless files
+    (setq files
+          (let* ((db-files (copy-hash-table org-mem--truename<>metadata))
+                 (modified-files
+                  (cl-loop
+                   for (truename . attr) in (org-mem--truenames-and-attrs)
+                   as real-mtime = (file-attribute-modification-time attr)
                    as db-mtime = (prog1 (org-mem-file-mtime truename)
                                    (remhash truename db-files))
                    when (or (not db-mtime)
                             (not (time-equal-p db-mtime real-mtime)))
                    collect truename))
-         (removed-files
-          (hash-table-keys db-files))
-         (all (append removed-files modified-files)))
-    ;; If async job already ongoing, this will cancel that and run a new one
+                 (removed-files
+                  (hash-table-keys db-files)))
+            (append removed-files modified-files))))
+  (when files
+    ;; If async job of ID `org-mem-updater' is already ongoing, this will
+    ;; cancel that and run a new one.  That's fine, because mtimes of those
+    ;; files being scanned won't have changed in our db (ongoing => results
+    ;; not yet written), so we've automatically picked them up again.
     (el-job-ng-run
      :id 'org-mem-updater
      :inject-vars (append (org-mem--mk-work-vars)
                           (el-job-ng-vars org-mem-inject-vars))
      :require (cons 'org-mem-parser org-mem-load-features)
-     :inputs all
+     :inputs files
      :funcall-per-input #'org-mem-parser--parse-file
      :callback #'org-mem-updater--finalize-targeted-scan)
     (when synchronous
       (el-job-ng-await-or-die
        'org-mem-updater 3600
-       (format "Running org-mem-updater-update... (files: %S)" all)))))
+       (format "Running org-mem-updater-update... (files: %S)" files)))))
 
 (defun org-mem-updater--finalize-targeted-scan (parse-results)
   "Handle PARSE-RESULTS from `org-mem-updater-update'."
@@ -79,7 +92,7 @@ If SYNCHRONOUS and interrupted by a quit, cancel the update."
   (let (problems)
     (org-mem--invalidate-file-names
      (cl-loop for (bad-path) in parse-results when bad-path collect bad-path))
-    (with-current-buffer (get-buffer-create " *org-mem-scratch*" t)
+    (with-current-buffer (get-buffer-create " *org-mem-fundamental-scratch*" t)
       (cl-loop for (_ problem file-datum entries links) in parse-results do
                (when problem (push problem problems))
                (let ((file (car file-datum)))
@@ -127,26 +140,21 @@ Override this if you prefer different timer delays, or no timer."
       (setq org-mem-updater--reset-timer
             (run-with-idle-timer new-delay t #'org-mem--scan-full)))))
 
-(defun org-mem-updater--update-file-soon (&optional file &rest _)
+(defvar org-mem-updater--debounce-timer nil)
+(defun org-mem-updater--update-soon (&optional file &rest _)
   "Schedule to run `org-mem-updater-update' very soon.
 
 Designed for `after-save-hook' and as advice for `delete-file' and
 `rename-file'.  Such functions might be called many times in a loop,
 and this design is meant to avoid invoking `org-mem-updater-update'
 for every FILE, but wait and do a massed invocation afterwards."
-  (setq file (or file buffer-file-name))
-  (when (and file (cl-some (##string-suffix-p % file) org-mem-suffixes))
-    (org-mem-updater--update-soon)))
-
-(defvar org-mem-updater--debounce-timer nil)
-(defun org-mem-updater--update-soon ()
-  "Schedule to run `org-mem-updater-update' very soon.
-If already scheduled, postpone to very soon."
-  (if (memq org-mem-updater--debounce-timer timer-list)
+  (setq file (or file (buffer-file-name (buffer-base-buffer))))
+  (when (and (stringp file) (cl-some (##string-suffix-p % file) org-mem-suffixes))
+    (if (memq org-mem-updater--debounce-timer timer-list)
       (timer-set-time org-mem-updater--debounce-timer
                       (time-add (current-time) 0.5))
     (setq org-mem-updater--debounce-timer
-          (run-with-timer 0.5 nil #'org-mem-updater-update))))
+          (run-with-timer 0.5 nil #'org-mem-updater-update)))))
 
 ;;;###autoload
 (define-minor-mode org-mem-updater-mode
@@ -157,15 +165,15 @@ If already scheduled, postpone to very soon."
   (if org-mem-updater-mode
       (progn
         (add-hook 'org-mem-post-full-scan-functions #'org-mem-updater-adjust-reset-timer 90)
-        (add-hook 'after-save-hook                  #'org-mem-updater--update-file-soon)
-        (advice-add 'rename-file :after             #'org-mem-updater--update-file-soon)
-        (advice-add 'delete-file :after             #'org-mem-updater--update-file-soon)
+        (add-hook 'after-save-hook                  #'org-mem-updater--update-soon)
+        (advice-add 'rename-file :after             #'org-mem-updater--update-soon)
+        (advice-add 'delete-file :after             #'org-mem-updater--update-soon)
         (org-mem-updater-adjust-reset-timer)
         (org-mem--scan-full))
     (remove-hook 'org-mem-post-full-scan-functions #'org-mem-updater-adjust-reset-timer)
-    (remove-hook 'after-save-hook                  #'org-mem-updater--update-file-soon)
-    (advice-remove 'rename-file                    #'org-mem-updater--update-file-soon)
-    (advice-remove 'delete-file                    #'org-mem-updater--update-file-soon)
+    (remove-hook 'after-save-hook                  #'org-mem-updater--update-soon)
+    (advice-remove 'rename-file                    #'org-mem-updater--update-soon)
+    (advice-remove 'delete-file                    #'org-mem-updater--update-soon)
     (cancel-timer org-mem-updater--reset-timer)))
 
 
