@@ -19,9 +19,9 @@
 ;; URL:      https://github.com/meedstrom/org-mem
 ;; Created:  2025-03-15
 ;; Keywords: text
-;; Package-Version: 20260224.1229
-;; Package-Revision: ebecc2378422
-;; Package-Requires: ((emacs "29.1") (el-job "2.7.3") (llama "1.0") (truename-cache "0.1.2"))
+;; Package-Version: 20260301.2353
+;; Package-Revision: fbe19417cacc
+;; Package-Requires: ((emacs "29.1") (el-job "2.7.3") (llama "1.0") (truename-cache "0.3.2"))
 
 ;;; Commentary:
 
@@ -42,7 +42,7 @@
 ;;; Code:
 
 (define-obsolete-variable-alias 'org-mem--bump-int 'org-mem-internal-version "2026-01-27 (after 0.26.4)")
-(defconst org-mem-internal-version 38 "Not a version number, but bumped sometimes.")
+(defconst org-mem-internal-version 41 "Not a version number, but bumped sometimes.")
 
 (require 'cl-lib)
 (require 'subr-x)
@@ -70,6 +70,9 @@
 (unless (and (boundp 'el-job-internal-version)
              (>= el-job-internal-version 107))
   (display-warning 'org-mem "Update to el-job 2.7.3+ to use this version of org-mem"))
+(unless (and (boundp 'truename-cache-internal-version)
+             (>= truename-cache-internal-version 12))
+  (display-warning 'org-mem "Update to truename-cache 0.3.2+ to use this version of org-mem"))
 
 (defgroup org-mem nil "Fast info from a large amount of Org file contents."
   :group 'org)
@@ -151,14 +154,17 @@ try command \\[org-mem-forget-id-locations-recursively]."
   :type '(repeat directory)
   :package-version '(org-mem . "0.5.0"))
 
-;; REVIEW: Use backslashes on Windows?
 (defcustom org-mem-exclude
   '("/logseq/bak/"
     "/logseq/version-files/"
     "/node_modules/"
     ".sync-conflict-" ;; Match e.g. "~/org/foo.sync-conflict-234234.org"
     "/backup" ;; Match e.g. "~/backups-2024/"
-    "/.#")
+    "/.#"
+    ;; These two are also hardcoded so it's moot, but hopefully we can stop
+    ;; hardcoding in the future.
+    "/."
+    "/_")
   "Literal substrings of file paths that should not be scanned.
 Aside from this variable, some filters are hard-coded:
 
@@ -175,7 +181,7 @@ You can also speed up `org-mem-reset' a bit by excluding directories
 found inside `org-mem-watch-dirs' with a very large amount of files
 \(on the order of 100,000), such as the infamous \"node_modules\"."
   :type '(repeat string)
-  :package-version '(org-mem . "0.15.1"))
+  :package-version '(org-mem . "0.32.0"))
 
 (defcustom org-mem-seek-link-types
   '("http" "https" "id" "file")
@@ -285,7 +291,7 @@ Note: All tables cleared often, meant for memoizations."
 (defun org-mem--get-file-metadata (file/entry/link)
   "Return list of assorted data if FILE/ENTRY/LINK known, else error."
   (let ((wild-file (if (stringp file/entry/link)
-                       (truename-cache-get-existed-p file/entry/link)
+                       (truename-cache-get-p file/entry/link)
                      (if (org-mem-entry-p file/entry/link)
                          (org-mem-entry-file-truename file/entry/link)
                        (if (org-mem-link-p file/entry/link)
@@ -324,7 +330,8 @@ Note: All tables cleared often, meant for memoizations."
   (tags-inherited        nil :read-only t :type list)
   (tags-local            nil :read-only t :type list)
   (todo-state            nil :read-only t :type string-or-nil)
-  (pseudo-id             -1  :read-only t :type integer        :documentation "Generated by `org-mem-parser--mk-id'.  Not `eq'-safe, use `=', `eql' or `equal'."))
+  (pseudo-id             -1  :read-only t :type integer        :documentation "Generated by `org-mem-parser--mk-id'.  Not `eq'-safe, use `=', `eql' or `equal'.")
+  (keywords              nil :read-only t :type list           :documentation "Alist of #+keywords: found in entry only.  See `org-mem-file-keywords'."))
 
 
 ;;; To find objects to operate on
@@ -977,17 +984,23 @@ Can be nil."
 (defun org-mem-file-known-p (file)
   "Return non-nil when FILE is known to org-mem.
 Specifically, return the name by which it is known.
-This is more restrictive than `truename-cache-get-existed-p'
+This is more restrictive than `truename-cache-get-p'
 by only returning non-nil if the file has been scanned by org-mem."
   (cl-assert (stringp file))
   (or (and (gethash file org-mem--truename<>entries)
            file)
-      (and (setq file (truename-cache-get-existed-p file))
+      (and (setq file (truename-cache-get-p file))
            (if (gethash file org-mem--truename<>entries)
                file
              (truename-cache-invalidate file)
              nil))))
 
+(defun org-mem-file-keywords (file)
+  "Alist of all #+keywords: in FILE, as if from `org-collect-keywords'."
+  (with-memoization (org-mem--table 56 file)
+    (org-mem-parser--merge-same-keywords
+     (seq-mapcat #'org-mem-entry-keywords (org-mem-entries-in-file file)))))
+
 
 ;;; Optional: Roam aliases and refs
 
@@ -1105,7 +1118,7 @@ These link-targets are determined by `org-mem--split-roam-refs-field'."
   "Extract valid components of a ROAM-REFS field.
 What is valid?  See \"org-mem-test.el\"."
   (when roam-refs
-    (with-current-buffer (get-buffer-create " *org-mem-scratch*" t)
+    (with-current-buffer (get-buffer-create " *org-mem-fundamental-scratch*" t)
       (erase-buffer)
       (insert roam-refs)
       (goto-char 1)
@@ -1234,12 +1247,14 @@ Like `org-mem-entry-title', this always returns a string."
   (:method ((xx org-mem-entry)) (org-mem-entry-title-maybe xx))
   (:method ((xx string)) (org-mem-file-title-strict xx)))
 
-(defun org-mem-entries-in (file/files)
-  "All entries in FILE/FILES."
-  (funcall (if (listp file/files)
-               #'org-mem-entries-in-files
-             #'org-mem-entries-in-file)
-           file/files))
+(defun org-mem-entries-in (file/files/entry)
+  "All entries in FILE/FILES/ENTRY."
+  (funcall (if (org-mem-entry-p file/files/entry)
+               #'org-mem-entry-children
+             (if (listp file/files/entry)
+                 #'org-mem-entries-in-files
+               #'org-mem-entries-in-file))
+           file/files/entry))
 
 
 ;;; Core logic
@@ -1371,11 +1386,12 @@ overrides a default message printed when `org-mem-do-cache-text' is t."
         problems)
     (org-mem--invalidate-file-names bad-paths)
     ;; Build tables.
-    (with-current-buffer (get-buffer-create " *org-mem-scratch*" t)
+    (with-current-buffer (get-buffer-create " *org-mem-fundamental-scratch*" t)
       (cl-loop for (_ problem file-data entries links) in parse-results do
                (when problem (push problem problems))
-               (puthash (car file-data) file-data org-mem--truename<>metadata)
-               (run-hook-with-args 'org-mem--record-file-functions file-data)
+               (when file-data
+                 (puthash (car file-data) file-data org-mem--truename<>metadata)
+                 (run-hook-with-args 'org-mem--record-file-functions file-data))
                (dolist (entry entries)
                  (org-mem--record-entry entry)
                  (run-hook-with-args 'org-mem--record-entry-functions entry))
@@ -1483,7 +1499,7 @@ Or no-op, if `org-mem-do-sync-with-org-id' is nil."
              (org-mem--try-ensure-org-id-table-p))
     (setq org-id-files (delete-dups (hash-table-values org-id-locations)))))
 
-(add-hook 'org-mem-post-full-scan-functions #'org-mem--reset-org-id-files)
+(add-hook 'org-mem-post-full-scan-functions #'org-mem--reset-org-id-files -10)
 
 (defun org-mem--mk-work-vars ()
   "Make alist of variables needed by `org-mem-parser--parse-file'."
@@ -1536,7 +1552,6 @@ Or no-op, if `org-mem-do-sync-with-org-id' is nil."
 
 (defun org-mem--invalidate-file-names (bad-names)
   "Scrub bad file names BAD-NAMES in the truename cache.
-Notably, invalidate part of the cache used by `org-mem-file-known-p'.
 If `org-mem-do-sync-with-org-id' t, also scrub `org-id-locations'."
   (let ((invalidated (seq-mapcat #'truename-cache-invalidate bad-names)))
     (when (and invalidated
@@ -1653,17 +1668,15 @@ corresponding to your package name."
 ;; to use with `org-mem-entry-text' to generate backlink previews, for
 ;; example, and it is apparently rare to realize the perf impact of
 ;; opening many Org buffers.
-;; Maybe if Org doesn't fix or can't fix the startup perf, they can ship an
-;; "org-scratch" function like this?  It belongs upstream.
-(defun org-mem-org-mode-scratch (&optional bufname)
+(defun org-mem-scratch (&optional bufname)
   "Get or create a hidden `org-mode' buffer.
 Ignore `org-mode-hook' and startup options.
 
 Like a temp buffer, but does not clean up.
 You should probably use `erase-buffer' in case it already contains text.
-BUFNAME defaults to \" *org-mem-org-mode-scratch*\"."
+BUFNAME defaults to \" *org-mem-scratch*\"."
   (require 'org)
-  (setq bufname (or bufname " *org-mem-org-mode-scratch*"))
+  (setq bufname (or bufname " *org-mem-scratch*"))
   (or (get-buffer bufname)
       (let ((org-inhibit-startup t)
             (org-agenda-files nil)
@@ -1676,13 +1689,15 @@ BUFNAME defaults to \" *org-mem-org-mode-scratch*\"."
 
 (defun org-mem-fontify-like-org (string)
   "Return STRING with text properties from fontifying it in `org-mode'."
-  (with-current-buffer (org-mem-org-mode-scratch)
+  (declare (obsolete nil "0.32.0 (2026-02-27)"))
+  (with-current-buffer (org-mem-scratch)
     (erase-buffer)
     (insert string)
     (font-lock-ensure)
     (buffer-string)))
 
 (defun org-mem-tip-if-empty ()
+  "If tables empty, print a helpful message."
   (when (hash-table-empty-p org-mem--truename<>metadata)
     (let ((msg (if org-mem-do-sync-with-org-id
                    (if (not (featurep 'org-id))
@@ -1722,7 +1737,6 @@ Old style was single list of 5 lists, representing all files combined:
    append links into all-links
    finally return (list badpaths fdata all-entries all-links problems)))
 
-
 
 ;;; End-user tool
 
@@ -1749,7 +1763,7 @@ org-id-locations:
   (require 'org-id)
   (let ((files (org-mem--dir-files-recursive dir org-mem-suffixes nil)))
     (when files
-      (setq files (append files (seq-keep #'truename-cache-get-existed-p files)))
+      (setq files (append files (seq-keep #'truename-cache-get-p files)))
       (message "Forgetting all IDs in directory %s..." dir)
       (redisplay t)
       (maphash (lambda (id file)
@@ -1815,9 +1829,10 @@ Still exists under name `org-mem--record-file-functions',
 but please use `org-mem-post-full-scan-functions'.")
 
 (define-obsolete-function-alias 'org-mem-entry-that-contains-link #'org-mem-link-entry "0.29.0 (2026-02-11)")
+(define-obsolete-function-alias 'org-mem-org-mode-scratch #'org-mem-scratch "0.32.0 (2026-02-27)")
 
 (defun org-mem--list-files-from-fs ()
-  (declare (obsolete nil "2026-02-16"))
+  (declare (obsolete nil "0.32.0 (2026-02-27)"))
   (mapcar #'car (org-mem--truenames-and-attrs)))
 
 (provide 'org-mem)
